@@ -5,9 +5,11 @@ use std::{mem, ptr};
 use glib::translate::*;
 use gst_base::{prelude::*, subclass::prelude::*};
 
-use crate::{subclass::AggregateFramesToken, VideoAggregator, VideoAggregatorPad};
+use crate::{ffi, subclass::AggregateFramesToken, VideoAggregator, VideoAggregatorPad};
 
-pub trait VideoAggregatorPadImpl: VideoAggregatorPadImplExt + AggregatorPadImpl {
+pub trait VideoAggregatorPadImpl:
+    AggregatorPadImpl + ObjectSubclass<Type: IsA<VideoAggregatorPad>>
+{
     fn update_conversion_info(&self) {
         self.parent_update_conversion_info()
     }
@@ -31,25 +33,7 @@ pub trait VideoAggregatorPadImpl: VideoAggregatorPadImplExt + AggregatorPadImpl
     }
 }
 
-pub trait VideoAggregatorPadImplExt: ObjectSubclass {
-    fn parent_update_conversion_info(&self);
-
-    fn parent_prepare_frame(
-        &self,
-        aggregator: &crate::VideoAggregator,
-        token: &AggregateFramesToken,
-        buffer: &gst::Buffer,
-    ) -> Option<crate::VideoFrame<crate::video_frame::Readable>>;
-
-    fn parent_clean_frame(
-        &self,
-        aggregator: &crate::VideoAggregator,
-        token: &AggregateFramesToken,
-        frame: Option<crate::VideoFrame<crate::video_frame::Readable>>,
-    );
-}
-
-impl<T: VideoAggregatorPadImpl> VideoAggregatorPadImplExt for T {
+pub trait VideoAggregatorPadImplExt: VideoAggregatorPadImpl {
     fn parent_update_conversion_info(&self) {
         unsafe {
             let data = Self::type_data();
@@ -70,10 +54,7 @@ impl<T: VideoAggregatorPadImpl> VideoAggregatorPadImplExt for T {
         token: &AggregateFramesToken,
         buffer: &gst::Buffer,
     ) -> Option<crate::VideoFrame<crate::video_frame::Readable>> {
-        assert_eq!(
-            aggregator.as_ptr() as *mut ffi::GstVideoAggregator,
-            token.0.as_ptr() as *mut ffi::GstVideoAggregator
-        );
+        assert_eq!(aggregator.as_ptr(), token.0.as_ptr());
 
         unsafe {
             let data = Self::type_data();
@@ -109,10 +90,7 @@ impl<T: VideoAggregatorPadImpl> VideoAggregatorPadImplExt for T {
         token: &AggregateFramesToken,
         frame: Option<crate::VideoFrame<crate::video_frame::Readable>>,
     ) {
-        assert_eq!(
-            aggregator.as_ptr() as *mut ffi::GstVideoAggregator,
-            token.0.as_ptr() as *mut ffi::GstVideoAggregator
-        );
+        assert_eq!(aggregator.as_ptr(), token.0.as_ptr());
 
         unsafe {
             let data = Self::type_data();
@@ -137,6 +115,8 @@ impl<T: VideoAggregatorPadImpl> VideoAggregatorPadImplExt for T {
     }
 }
 
+impl<T: VideoAggregatorPadImpl> VideoAggregatorPadImplExt for T {}
+
 unsafe impl<T: VideoAggregatorPadImpl> IsSubclassable<T> for VideoAggregatorPad {
     fn class_init(klass: &mut glib::Class<Self>) {
         Self::parent_class_init::<T>(klass);
