@@ -2,6 +2,7 @@
 
 use std::{fmt, ptr};
 
+use crate::ffi;
 use glib::translate::*;
 use gst::prelude::*;
 
@@ -20,13 +21,18 @@ impl VideoMeta {
         format: crate::VideoFormat,
         width: u32,
         height: u32,
-    ) -> Result<gst::MetaRefMut<Self, gst::meta::Standalone>, glib::BoolError> {
+    ) -> Result<gst::MetaRefMut<'_, Self, gst::meta::Standalone>, glib::BoolError> {
         skip_assert_initialized!();
 
         if format == crate::VideoFormat::Unknown || format == crate::VideoFormat::Encoded {
             return Err(glib::bool_error!("Unsupported video format {}", format));
         }
 
+        #[cfg(feature = "v1_24")]
+        if format == crate::VideoFormat::DmaDrm {
+            return Err(glib::bool_error!("Use `add_full()` for DMA_DRM formats"));
+        }
+
         let info = crate::VideoInfo::builder(format, width, height).build()?;
 
         if !info.is_valid() {
@@ -73,11 +79,48 @@ impl VideoMeta {
             return Err(glib::bool_error!("Unsupported video format {}", format));
         }
 
-        let n_planes = offset.len() as u32;
-        let info = crate::VideoInfo::builder(format, width, height)
-            .offset(offset)
-            .stride(stride)
-            .build()?;
+        assert_eq!(offset.len(), stride.len());
+
+        unsafe {
+            let meta = ffi::gst_buffer_add_video_meta_full(
+                buffer.as_mut_ptr(),
+                video_frame_flags.into_glib(),
+                format.into_glib(),
+                width,
+                height,
+                offset.len() as u32,
+                offset.as_ptr() as *mut _,
+                stride.as_ptr() as *mut _,
+            );
+
+            if meta.is_null() {
+                return Err(glib::bool_error!("Failed to add video meta"));
+            }
+
+            Ok(Self::from_mut_ptr(buffer, meta))
+        }
+    }
+
+    pub fn add_from_info<'a>(
+        buffer: &'a mut gst::BufferRef,
+        video_frame_flags: crate::VideoFrameFlags,
+        info: &crate::VideoInfo,
+    ) -> Result<gst::MetaRefMut<'a, Self, gst::meta::Standalone>, glib::BoolError> {
+        skip_assert_initialized!();
+
+        if info.format() == crate::VideoFormat::Unknown
+            || info.format() == crate::VideoFormat::Encoded
+        {
+            return Err(glib::bool_error!(
+                "Unsupported video format {}",
+                info.format()
+            ));
+        }
+
+        #[cfg(feature = "v1_24")]
+        if info.format() == crate::VideoFormat::DmaDrm {
+            return Err(glib::bool_error!("Use `add_full()` for DMA_DRM formats"));
+        }
 
         if !info.is_valid() {
             return Err(glib::bool_error!("Invalid video info"));
@@ -91,24 +134,15 @@ impl VideoMeta {
             ));
         }
 
-        unsafe {
-            let meta = ffi::gst_buffer_add_video_meta_full(
-                buffer.as_mut_ptr(),
-                video_frame_flags.into_glib(),
-                format.into_glib(),
-                width,
-                height,
-                n_planes,
-                offset.as_ptr() as *mut _,
-                stride.as_ptr() as *mut _,
-            );
-
-            if meta.is_null() {
-                return Err(glib::bool_error!("Failed to add video meta"));
-            }
-
-            Ok(Self::from_mut_ptr(buffer, meta))
-        }
+        Self::add_full(
+            buffer,
+            video_frame_flags,
+            info.format(),
+            info.width(),
+            info.height(),
+            info.offset(),
+            info.stride(),
+        )
     }
 
     #[doc(alias = "get_flags")]
@@ -260,7 +294,7 @@ impl VideoCropMeta {
     pub fn add(
         buffer: &mut gst::BufferRef,
         rect: (u32, u32, u32, u32),
-    ) -> gst::MetaRefMut<Self, gst::meta::Standalone> {
+    ) -> gst::MetaRefMut<'_, Self, gst::meta::Standalone> {
         skip_assert_initialized!();
         unsafe {
             let meta = gst::ffi::gst_buffer_add_meta(
@@ -368,7 +402,7 @@ impl VideoRegionOfInterestMeta {
     }
 
     #[doc(alias = "get_params")]
-    pub fn params(&self) -> ParamsIter {
+    pub fn params(&self) -> ParamsIter<'_> {
         ParamsIter {
             _meta: self,
             list: ptr::NonNull::new(self.0.params),
@@ -407,6 +441,7 @@ impl VideoRegionOfInterestMeta {
     }
 }
 
+#[must_use = "iterators are lazy and do nothing unless consumed"]
 pub struct ParamsIter<'a> {
     _meta: &'a VideoRegionOfInterestMeta,
     list: Option<ptr::NonNull<glib::ffi::GList>>,
@@ -430,7 +465,7 @@ impl<'a> Iterator for ParamsIter<'a> {
     }
 }
 
-impl<'a> std::iter::FusedIterator for ParamsIter<'a> {}
+impl std::iter::FusedIterator for ParamsIter<'_> {}
 
 unsafe impl MetaAPI for VideoRegionOfInterestMeta {
     type GstType = ffi::GstVideoRegionOfInterestMeta;
@@ -694,7 +729,7 @@ impl VideoAFDMeta {
         field: u8,
         spec: crate::VideoAFDSpec,
         afd: crate::VideoAFDValue,
-    ) -> gst::MetaRefMut<Self, gst::meta::Standalone> {
+    ) -> gst::MetaRefMut<'_, Self, gst::meta::Standalone> {
         skip_assert_initialized!();
 
         unsafe {
@@ -775,7 +810,7 @@ impl VideoBarMeta {
         is_letterbox: bool,
         bar_data1: u32,
         bar_data2: u32,
-    ) -> gst::MetaRefMut<Self, gst::meta::Standalone> {
+    ) -> gst::MetaRefMut<'_, Self, gst::meta::Standalone> {
         skip_assert_initialized!();
 
         unsafe {
@@ -860,7 +895,7 @@ impl VideoCodecAlphaMeta {
     pub fn add(
         buffer: &mut gst::BufferRef,
         alpha_buffer: gst::Buffer,
-    ) -> gst::MetaRefMut<Self, gst::meta::Standalone> {
+    ) -> gst::MetaRefMut<'_, Self, gst::meta::Standalone> {
         skip_assert_initialized!();
         unsafe {
             let meta = ffi::gst_buffer_add_video_codec_alpha_meta(
@@ -1007,7 +1042,7 @@ unsafe impl Sync for AncillaryMeta {}
 #[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
 impl AncillaryMeta {
     #[doc(alias = "gst_buffer_add_ancillary_meta")]
-    pub fn add(buffer: &mut gst::BufferRef) -> gst::MetaRefMut<Self, gst::meta::Standalone> {
+    pub fn add(buffer: &mut gst::BufferRef) -> gst::MetaRefMut<'_, Self, gst::meta::Standalone> {
         skip_assert_initialized!();
         unsafe {
             let meta = ffi::gst_buffer_add_ancillary_meta(buffer.as_mut_ptr());
@@ -1117,11 +1152,9 @@ impl AncillaryMeta {
 
     #[inline]
     pub fn set_data(&mut self, data: glib::Slice<u16>) {
-        unsafe {
-            assert!(data.len() < 256);
-            self.0.data_count = data.len() as u16;
-            self.0.data = data.into_glib_ptr();
-        }
+        assert!(data.len() < 256);
+        self.0.data_count = data.len() as u16;
+        self.0.data = data.into_glib_ptr();
     }
 
     #[inline]
@@ -1161,6 +1194,45 @@ impl fmt::Debug for AncillaryMeta {
     }
 }
 
+pub mod tags {
+    gst::impl_meta_tag!(Video, crate::ffi::GST_META_TAG_VIDEO_STR);
+    gst::impl_meta_tag!(Size, crate::ffi::GST_META_TAG_VIDEO_SIZE_STR);
+    gst::impl_meta_tag!(Orientation, crate::ffi::GST_META_TAG_VIDEO_ORIENTATION_STR);
+    gst::impl_meta_tag!(Colorspace, crate::ffi::GST_META_TAG_VIDEO_COLORSPACE_STR);
+}
+
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct VideoMetaTransformScale<'a> {
+    in_info: &'a crate::VideoInfo,
+    out_info: &'a crate::VideoInfo,
+}
+
+impl<'a> VideoMetaTransformScale<'a> {
+    pub fn new(in_info: &'a crate::VideoInfo, out_info: &'a crate::VideoInfo) -> Self {
+        skip_assert_initialized!();
+        VideoMetaTransformScale { in_info, out_info }
+    }
+}
+
+unsafe impl<'a> gst::meta::MetaTransform<'a> for VideoMetaTransformScale<'a> {
+    type GLibType = ffi::GstVideoMetaTransform;
+
+    #[doc(alias = "gst_video_meta_transform_scale_get_quark")]
+    fn quark() -> glib::Quark {
+        unsafe { from_glib(ffi::gst_video_meta_transform_scale_get_quark()) }
+    }
+
+    fn to_raw<T: MetaAPI>(
+        &self,
+        _meta: &gst::MetaRef<T>,
+    ) -> Result<ffi::GstVideoMetaTransform, glib::BoolError> {
+        Ok(ffi::GstVideoMetaTransform {
+            in_info: mut_override(self.in_info.to_glib_none().0),
+            out_info: mut_override(self.out_info.to_glib_none().0),
+        })
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1187,6 +1259,10 @@ mod tests {
             assert_eq!(meta.n_planes(), 1);
             assert_eq!(meta.offset(), &[0]);
             assert_eq!(meta.stride(), &[320 * 4]);
+            assert!(meta.has_tag::<gst::meta::tags::Memory>());
+            assert!(meta.has_tag::<tags::Video>());
+            assert!(meta.has_tag::<tags::Colorspace>());
+            assert!(meta.has_tag::<tags::Size>());
         }
 
         {
@@ -1241,9 +1317,26 @@ mod tests {
         }
     }
 
+    #[test]
+    #[cfg(feature = "v1_16")]
+    fn test_add_full_alternate_interlacing() {
+        gst::init().unwrap();
+        let mut buffer = gst::Buffer::with_size(320 * 120 * 4).unwrap();
+        VideoMeta::add_full(
+            buffer.get_mut().unwrap(),
+            crate::VideoFrameFlags::TOP_FIELD,
+            crate::VideoFormat::Argb,
+            320,
+            240,
+            &[0],
+            &[320 * 4],
+        )
+        .unwrap();
+    }
+
     #[test]
     #[cfg(feature = "v1_18")]
-    fn test_vide_meta_alignment() {
+    fn test_video_meta_alignment() {
         gst::init().unwrap();
 
         let mut buffer = gst::Buffer::with_size(115200).unwrap();
@@ -1354,4 +1447,31 @@ mod tests {
         assert_eq!(meta.data(), META_DATA);
         assert_eq!(meta.data().len(), META_DATA.len());
     }
+
+    #[test]
+    fn test_meta_video_transform() {
+        gst::init().unwrap();
+
+        let mut buffer = gst::Buffer::with_size(320 * 240 * 4).unwrap();
+        let meta = VideoCropMeta::add(buffer.get_mut().unwrap(), (10, 10, 20, 20));
+
+        let mut buffer2 = gst::Buffer::with_size(640 * 480 * 4).unwrap();
+
+        let in_video_info = crate::VideoInfo::builder(crate::VideoFormat::Rgba, 320, 240)
+            .build()
+            .unwrap();
+        let out_video_info = crate::VideoInfo::builder(crate::VideoFormat::Rgba, 640, 480)
+            .build()
+            .unwrap();
+
+        meta.transform(
+            buffer2.get_mut().unwrap(),
+            &VideoMetaTransformScale::new(&in_video_info, &out_video_info),
+        )
+        .unwrap();
+
+        let meta2 = buffer2.meta::<VideoCropMeta>().unwrap();
+
+        assert_eq!(meta2.rect(), (20, 20, 40, 40));
+    }
 }
